Package: SparseGrid
Type: Package
Title: Sparse grid integration in R
Version: 0.8.2
Date: 2012-07-31
Author: Jelmer Ypma
Maintainer: Jelmer Ypma <uctpjyy@ucl.ac.uk>
Suggests: testthat, statmod, mvtnorm
Description: SparseGrid is a package to create sparse grids for numerical integration, based on code from www.sparse-grids.de
License: GPL
Packaged: 2013-07-31 18:18:25 UTC; Jelmer
NeedsCompilation: no
Repository: CRAN
Date/Publication: 2013-07-31 21:18:24
Built: R 4.6.0; ; 2026-04-26 12:44:20 UTC; unix
